@@ -2,46 +2,41 @@
 
 use std::fs;
 use std::path::PathBuf;
+use std::sync::Arc;
 
+use axum::handler::Handler;
+use axum::routing::{get, post};
+use axum::{Extension, Router};
 #[cfg(target_family = "unix")]
 use pnet::datalink;
 use qr2term::matrix::Matrix;
 use qr2term::qr::Qr;
 use qr2term::render::{Color, QrDark, QrLight, Renderer};
-use rocket::config::{Config, Environment};
-use rocket::Catcher;
+use tower::ServiceBuilder;
+use tower_http::trace::TraceLayer;
 
-use crate::error::QrSyncError;
-use crate::routes::{
-    bad_request, static_rocket_route_info_for_get_done, static_rocket_route_info_for_get_error,
-    static_rocket_route_info_for_get_receive, static_rocket_route_info_for_get_send,
-    static_rocket_route_info_for_post_receive, static_rocket_route_info_for_slash,
-    static_rocket_route_info_for_static_bootstrap_css,
-    static_rocket_route_info_for_static_bootstrap_css_map,
-    static_rocket_route_info_for_static_favicon, RequestCtx,
-};
-use crate::ResultOrError;
+use crate::routes::*;
+use crate::{QrSyncError, QrSyncResult};
 
 /// Main structure implementing the workflow if sending and receving files between devices.
 /// It fetches the main IP address, generates the QR code, configures and runs the Rocket worker.
-#[allow(dead_code)]
+#[derive(Debug)]
 pub struct QrSyncHttp {
     ip_address: Option<String>,
     port: u16,
     filename: Option<String>,
     root_dir: PathBuf,
-    workers: u16,
     light_term: bool,
     ipv6: bool,
 }
 
 impl QrSyncHttp {
+    /// Create a new instance of QrSyncHttp from command line arguments.
     pub fn new(
         ip_address: Option<String>,
         port: u16,
         filename: Option<String>,
         root_dir: PathBuf,
-        workers: u16,
         light_term: bool,
         ipv6: bool,
     ) -> Self {
@@ -50,7 +45,6 @@ impl QrSyncHttp {
             port,
             filename,
             root_dir,
-            workers,
             light_term,
             ipv6,
         }
@@ -60,7 +54,7 @@ impl QrSyncHttp {
     /// routable interface with an IP address which can be reached from the outside.
     /// This method currently works only on *nix.
     #[cfg(target_family = "unix")]
-    fn find_public_ip(&self) -> ResultOrError<String> {
+    fn find_public_ip(&self) -> QrSyncResult<String> {
         if self.ip_address.is_some() {
             return Ok(self.ip_address.as_ref().unwrap().to_string());
         }
@@ -74,7 +68,7 @@ impl QrSyncHttp {
             Some(interface) => {
                 for ip in interface.ips.iter() {
                     if self.ipv6 {
-                        if ip.is_ipv6() && ip.ip().is_global() {
+                        if ip.is_ipv6() {
                             ip_address = ip.ip();
                             break;
                         }
@@ -84,22 +78,17 @@ impl QrSyncHttp {
                     }
                 }
                 if !ip_address.is_loopback() {
-                    debug!(
-                        "Found IP address {} for interface {}",
-                        ip_address, interface.name
-                    );
+                    tracing::debug!("Found IP address {} for interface {}", ip_address, interface.name);
                     Ok(ip_address.to_string())
                 } else {
-                    Err(QrSyncError::new(
-                        "Unable to find a valid IP address to bind with. See --ip-address option to specify the IP address to use", 
-                        Some("ip-discovery")
+                    Err(QrSyncError::Error(
+                        "Unable to find a valid IP address to bind with. See --ip-address option to specify the IP address to use".into() 
                     ))
                 }
-            },
-            None => Err(QrSyncError::new(
-                "Unable to find default interface. See --ip-address option to specify the IP address to use",
-                Some("ip-discovery"),
-            ))
+            }
+            None => Err(QrSyncError::Error(
+                "Unable to find default interface. See --ip-address option to specify the IP address to use".into(),
+            )),
         }
     }
 
@@ -108,7 +97,7 @@ impl QrSyncHttp {
     /// platform.
     /// This method currently works only on windows.
     #[cfg(target_family = "windows")]
-    fn find_public_ip(&self) -> ResultOrError<String> {
+    fn find_public_ip(&self) -> QrSyncResult<String> {
         match &self.ip_address {
             Some(ip_address) => Ok(ip_address.to_string()),
             None => Err(QrSyncError::new(
@@ -120,37 +109,32 @@ impl QrSyncHttp {
 
     /// Generates the QR code based on the mode QrSync is started, giving the user a different URL
     /// in case we are expecting the mobile device to send to receive the file.
-    fn generate_qr_code_url(&self, ip_address: String) -> ResultOrError<String> {
-        let url: String;
-        if self.filename.is_some() {
-            let filename = self.filename.as_ref().unwrap();
-            info!(
-                "Send mode enabled for file {}",
-                fs::canonicalize(filename)?.display()
-            );
-            url = format!(
-                "http://{}:{}/{}",
-                ip_address,
-                self.port,
-                base64::encode_config(filename, base64::URL_SAFE_NO_PAD)
-            );
-        } else {
-            info!(
-                "Receive mode enabled inside directory {}",
-                fs::canonicalize(&self.root_dir)?.display()
-            );
-            url = format!("http://{}:{}/receive", ip_address, self.port);
+    fn generate_qr_code_url(&self, ip_address: &str) -> QrSyncResult<String> {
+        let url = match self.filename.as_ref() {
+            Some(filename) => {
+                tracing::info!("Send mode enabled for file {}", fs::canonicalize(filename)?.display());
+                format!(
+                    "http://{}:{}/{}",
+                    ip_address,
+                    self.port,
+                    base64::encode_config(filename, base64::URL_SAFE_NO_PAD)
+                )
+            }
+            None => {
+                tracing::info!(
+                    "Receive mode enabled inside directory {}",
+                    fs::canonicalize(&self.root_dir)?.display()
+                );
+                format!("http://{}:{}/receive", ip_address, self.port)
+            }
         };
-        info!(
-            "Scan this QR code with a QR code reader app to open the URL {}",
-            url
-        );
+        tracing::info!("Scan this QR code with a QR code reader app to open the URL {}", url);
         Ok(url)
     }
 
     /// Print the QR code to stdout on the terminal and generates white based QRs on dark terminals
     /// and black based QRs on light terminals.
-    fn generate_qr_code_matrix(&self, data: &str) -> ResultOrError<Matrix<Color>> {
+    fn generate_qr_code_matrix(&self, data: &str) -> QrSyncResult<Matrix<Color>> {
         let mut matrix = Qr::from(data)?.to_matrix();
         if self.light_term {
             matrix.surround(2, QrDark);
@@ -160,60 +144,41 @@ impl QrSyncHttp {
         Ok(matrix)
     }
 
-    fn print_qr_code(&self, ip_address: String) -> ResultOrError<()> {
+    fn print_qr_code(&self, ip_address: &str) -> QrSyncResult<()> {
         let url = self.generate_qr_code_url(ip_address)?;
         let qr = self.generate_qr_code_matrix(&url)?;
         Renderer::default().print_stdout(&qr);
         Ok(())
     }
 
-    /// Build a list of Rocket::Catcher for any HTTP error Rocket supports to allow presenting a
-    /// nice page to the user.
-    fn build_error_catchers(&self) -> Vec<Catcher> {
-        let codes = vec![
-            400, 401, 402, 403, 404, 405, 406, 407, 408, 409, 410, 411, 412, 413, 414, 415, 416,
-            417, 418, 421, 426, 428, 429, 431, 451, 500, 501, 503, 511,
-        ];
-        let mut catchers: Vec<Catcher> = Vec::new();
-        for code in codes.iter() {
-            catchers.push(Catcher::new(*code, bad_request));
-        }
-        catchers
-    }
-
-    /// Configure rocket, print the QR code and run the HTTP worker.
-    pub fn run(&self) -> ResultOrError<()> {
+    /// Configure Axum, print the QR code and run the HTTP worker.
+    pub async fn run(&self) -> QrSyncResult<()> {
+        let app = Router::new()
+            .route("/", get(slash))
+            .route("/receive", get(get_receive))
+            .route("/receive_done", get(get_receive_done))
+            .route("/error", get(get_error))
+            .route("/static/bootstrap.min.css", get(static_bootstrap_css))
+            .route("/static/bootstrap.min.css.map", get(static_bootstrap_css_map))
+            .route("/favicon.ico", get(static_favicon))
+            .route("/:file_name", get(get_send))
+            .route("/receive", post(post_receive))
+            .fallback(bad_request.into_service());
+        let state = Arc::new(State::new(self.filename.clone(), &self.root_dir));
+        let app = app.layer(
+            ServiceBuilder::new()
+                .layer(TraceLayer::new_for_http())
+                .layer(Extension(state)),
+        );
         let ip_address = self.find_public_ip()?;
-        let config = Config::build(Environment::Production)
-            .address(&ip_address)
-            .port(self.port)
-            .workers(self.workers)
-            .finalize()?;
-        let app = rocket::custom(config);
-        self.print_qr_code(ip_address)?;
-        let error = app
-            .mount(
-                "/",
-                routes![
-                    slash,
-                    get_error,
-                    get_send,
-                    get_receive,
-                    get_done,
-                    post_receive,
-                    static_bootstrap_css,
-                    static_bootstrap_css_map,
-                    static_favicon,
-                ],
-            )
-            .register(self.build_error_catchers())
-            .manage(RequestCtx::new(self.filename.clone(), &self.root_dir))
-            .launch();
+        self.print_qr_code(&ip_address)?;
+        let address = format!("{}:{}", ip_address, self.port).parse()?;
+        let server = hyper::Server::bind(&address).serve(app.into_make_service());
 
-        Err(QrSyncError::new(
-            &format!("Launch failed! Error: {}", error),
-            Some("rocket-launch"),
-        ))
+        if let Err(err) = server.await {
+            tracing::error!("Server error: {}", err);
+        }
+        Ok(())
     }
 }
 
@@ -230,7 +195,6 @@ mod test {
             12345,
             Some("a-file".to_string()),
             PathBuf::from("a-dir"),
-            1,
             false,
             false,
         );
@@ -244,7 +208,6 @@ mod test {
             12345,
             Some("a-file".to_string()),
             PathBuf::from("a-dir"),
-            1,
             false,
             false,
         );
@@ -260,11 +223,10 @@ mod test {
             12345,
             Some(file_name.to_string()),
             PathBuf::from("a-dir"),
-            1,
             false,
             false,
         );
-        let url = http.generate_qr_code_url(ip_address.to_string()).unwrap();
+        let url = http.generate_qr_code_url(ip_address).unwrap();
         assert_eq!(
             format!(
                 "http://{}:12345/{}",
@@ -283,11 +245,10 @@ mod test {
             12345,
             None,
             PathBuf::from("a-dir"),
-            1,
             false,
             false,
         );
-        let url = http.generate_qr_code_url(ip_address.to_string()).unwrap();
+        let url = http.generate_qr_code_url(ip_address).unwrap();
         assert_eq!(format!("http://{}:12345/receive", ip_address,), url);
     }
 
@@ -299,11 +260,10 @@ mod test {
             12345,
             None,
             PathBuf::from("a-dir"),
-            1,
             false,
             false,
         );
-        let url = http.generate_qr_code_url(ip_address.to_string()).unwrap();
+        let url = http.generate_qr_code_url(ip_address).unwrap();
         let qr = http.generate_qr_code_matrix(&url).unwrap();
         assert_eq!(qr.pixels().len(), 1089);
         let light_pixels = qr.pixels().iter().filter(|&n| *n == QrLight).count();
@@ -320,11 +280,10 @@ mod test {
             12345,
             None,
             PathBuf::from("a-dir"),
-            1,
             true,
             false,
         );
-        let url = http.generate_qr_code_url(ip_address.to_string()).unwrap();
+        let url = http.generate_qr_code_url(ip_address).unwrap();
         let qr = http.generate_qr_code_matrix(&url).unwrap();
         assert_eq!(qr.pixels().len(), 1089);
         let light_pixels = qr.pixels().iter().filter(|&n| *n == QrLight).count();
@@ -333,22 +292,6 @@ mod test {
         assert_eq!(dark_pixels, 670);
     }
 
-    #[test]
-    fn test_build_error_catcher() {
-        let ip_address = "10.0.0.1";
-        let http = QrSyncHttp::new(
-            Some(ip_address.to_string()),
-            12345,
-            None,
-            PathBuf::from("a-dir"),
-            1,
-            false,
-            false,
-        );
-        let catchers = http.build_error_catchers();
-        assert_eq!(catchers.len(), 29);
-    }
-
     #[test]
     fn test_print_qr_code() {
         let ip_address = "10.0.0.1";
@@ -357,10 +300,9 @@ mod test {
             12345,
             None,
             PathBuf::from("a-dir"),
-            1,
             false,
             false,
         );
-        assert_eq!(http.print_qr_code(ip_address.to_string()).is_ok(), true);
+        assert_eq!(http.print_qr_code(ip_address).is_ok(), true);
     }
 }
